@@ -61,14 +61,14 @@ static struct flist_head *cgroup_list;
 static char *cgroup_mnt;
 static int exit_value;
 static volatile int fio_abort;
+static unsigned int nr_process = 0;
+static unsigned int nr_thread = 0;
 
 struct io_log *agg_io_log[DDIR_RWDIR_CNT];
 
 int groupid = 0;
 unsigned int thread_number = 0;
 unsigned int stat_number = 0;
-unsigned int nr_process = 0;
-unsigned int nr_thread = 0;
 int shm_id = 0;
 int temp_stall_ts;
 unsigned long done_secs = 0;
@@ -835,7 +835,7 @@ sync_done:
 	check_update_rusage(td);
 
 	if (td->trim_entries)
-		log_err("fio: %d trim entries leaked?\n", td->trim_entries);
+		log_err("fio: %lu trim entries leaked?\n", td->trim_entries);
 
 	if (td->o.fill_device && td->error == ENOSPC) {
 		td->error = 0;
@@ -910,6 +910,15 @@ static int init_io_u(struct thread_data *td)
 	if ((td->io_ops->flags & FIO_NOIO) || !(td_read(td) || td_write(td)))
 		data_xfer = 0;
 
+	/*
+	 * if we may later need to do address alignment, then add any
+	 * possible adjustment here so that we don't cause a buffer
+	 * overflow later. this adjustment may be too much if we get
+	 * lucky and the allocator gives us an aligned address.
+	 */
+	if (td->o.odirect || td->o.mem_align || (td->io_ops->flags & FIO_RAWIO))
+		td->orig_buffer_size += page_mask + td->o.mem_align;
+
 	if (td->o.mem_type == MEM_SHMHUGE || td->o.mem_type == MEM_MMAPHUGE) {
 		unsigned long bs;
 
@@ -1092,19 +1101,24 @@ static void *thread_main(void *data)
 {
 	unsigned long long elapsed;
 	struct thread_data *td = data;
+	struct thread_options *o = &td->o;
 	pthread_condattr_t attr;
 	int clear_state;
+	int ret;
 
-	if (!td->o.use_thread) {
+	if (!o->use_thread) {
 		setsid();
 		td->pid = getpid();
 	} else
 		td->pid = gettid();
 
-	fio_local_clock_init(td->o.use_thread);
+	fio_local_clock_init(o->use_thread);
 
 	dprint(FD_PROCESS, "jobs pid=%d started\n", (int) td->pid);
 
+	if (is_backend)
+		fio_server_send_start(td);
+
 	INIT_FLIST_HEAD(&td->io_u_freelist);
 	INIT_FLIST_HEAD(&td->io_u_busylist);
 	INIT_FLIST_HEAD(&td->io_u_requeues);
@@ -1132,16 +1146,17 @@ static void *thread_main(void *data)
 	 * eating a file descriptor
 	 */
 	fio_mutex_remove(td->mutex);
+	td->mutex = NULL;
 
 	/*
 	 * A new gid requires privilege, so we need to do this before setting
 	 * the uid.
 	 */
-	if (td->o.gid != -1U && setgid(td->o.gid)) {
+	if (o->gid != -1U && setgid(o->gid)) {
 		td_verror(td, errno, "setgid");
 		goto err;
 	}
-	if (td->o.uid != -1U && setuid(td->o.uid)) {
+	if (o->uid != -1U && setuid(o->uid)) {
 		td_verror(td, errno, "setuid");
 		goto err;
 	}
@@ -1150,21 +1165,24 @@ static void *thread_main(void *data)
 	 * If we have a gettimeofday() thread, make sure we exclude that
 	 * thread from this job
 	 */
-	if (td->o.gtod_cpu)
-		fio_cpu_clear(&td->o.cpumask, td->o.gtod_cpu);
+	if (o->gtod_cpu)
+		fio_cpu_clear(&o->cpumask, o->gtod_cpu);
 
 	/*
 	 * Set affinity first, in case it has an impact on the memory
 	 * allocations.
 	 */
-	if (td->o.cpumask_set && fio_setaffinity(td->pid, td->o.cpumask) == -1) {
-		td_verror(td, errno, "cpu_set_affinity");
-		goto err;
+	if (o->cpumask_set) {
+		ret = fio_setaffinity(td->pid, o->cpumask);
+		if (ret == -1) {
+			td_verror(td, errno, "cpu_set_affinity");
+			goto err;
+		}
 	}
 
 #ifdef CONFIG_LIBNUMA
 	/* numa node setup */
-	if (td->o.numa_cpumask_set || td->o.numa_memmask_set) {
+	if (o->numa_cpumask_set || o->numa_memmask_set) {
 		int ret;
 
 		if (numa_available() < 0) {
@@ -1172,8 +1190,8 @@ static void *thread_main(void *data)
 			goto err;
 		}
 
-		if (td->o.numa_cpumask_set) {
-			ret = numa_run_on_node_mask(td->o.numa_cpunodesmask);
+		if (o->numa_cpumask_set) {
+			ret = numa_run_on_node_mask(o->numa_cpunodesmask);
 			if (ret == -1) {
 				td_verror(td, errno, \
 					"numa_run_on_node_mask failed\n");
@@ -1181,20 +1199,20 @@ static void *thread_main(void *data)
 			}
 		}
 
-		if (td->o.numa_memmask_set) {
+		if (o->numa_memmask_set) {
 
-			switch (td->o.numa_mem_mode) {
+			switch (o->numa_mem_mode) {
 			case MPOL_INTERLEAVE:
-				numa_set_interleave_mask(td->o.numa_memnodesmask);
+				numa_set_interleave_mask(o->numa_memnodesmask);
 				break;
 			case MPOL_BIND:
-				numa_set_membind(td->o.numa_memnodesmask);
+				numa_set_membind(o->numa_memnodesmask);
 				break;
 			case MPOL_LOCAL:
 				numa_set_localalloc();
 				break;
 			case MPOL_PREFERRED:
-				numa_set_preferred(td->o.numa_mem_prefer_node);
+				numa_set_preferred(o->numa_mem_prefer_node);
 				break;
 			case MPOL_DEFAULT:
 			default:
@@ -1205,6 +1223,9 @@ static void *thread_main(void *data)
 	}
 #endif
 
+	if (fio_pin_memory(td))
+		goto err;
+
 	/*
 	 * May alter parameters that init_io_u() will use, so we need to
 	 * do this first.
@@ -1215,29 +1236,30 @@ static void *thread_main(void *data)
 	if (init_io_u(td))
 		goto err;
 
-	if (td->o.verify_async && verify_async_init(td))
+	if (o->verify_async && verify_async_init(td))
 		goto err;
 
-	if (td->ioprio_set) {
-		if (ioprio_set(IOPRIO_WHO_PROCESS, 0, td->ioprio) == -1) {
+	if (o->ioprio) {
+		ret = ioprio_set(IOPRIO_WHO_PROCESS, 0, o->ioprio_class, o->ioprio);
+		if (ret == -1) {
 			td_verror(td, errno, "ioprio_set");
 			goto err;
 		}
 	}
 
-	if (td->o.cgroup && cgroup_setup(td, cgroup_list, &cgroup_mnt))
+	if (o->cgroup && cgroup_setup(td, cgroup_list, &cgroup_mnt))
 		goto err;
 
 	errno = 0;
-	if (nice(td->o.nice) == -1 && errno != 0) {
+	if (nice(o->nice) == -1 && errno != 0) {
 		td_verror(td, errno, "nice");
 		goto err;
 	}
 
-	if (td->o.ioscheduler && switch_ioscheduler(td))
+	if (o->ioscheduler && switch_ioscheduler(td))
 		goto err;
 
-	if (!td->o.create_serialize && setup_files(td))
+	if (!o->create_serialize && setup_files(td))
 		goto err;
 
 	if (td_io_init(td))
@@ -1246,16 +1268,16 @@ static void *thread_main(void *data)
 	if (init_random_map(td))
 		goto err;
 
-	if (td->o.exec_prerun) {
-		if (exec_string(td->o.exec_prerun))
-			goto err;
-	}
+	if (o->exec_prerun && exec_string(o->exec_prerun))
+		goto err;
 
-	if (td->o.pre_read) {
+	if (o->pre_read) {
 		if (pre_read_files(td) < 0)
 			goto err;
 	}
 
+	fio_verify_init(td);
+
 	fio_gettime(&td->epoch, NULL);
 	fio_getrusage(&td->ru_start);
 	clear_state = 0;
@@ -1267,8 +1289,8 @@ static void *thread_main(void *data)
 		memcpy(&td->iops_sample_time, &td->start, sizeof(td->start));
 		memcpy(&td->tv_cache, &td->start, sizeof(td->start));
 
-		if (td->o.ratemin[DDIR_READ] || td->o.ratemin[DDIR_WRITE] ||
-				td->o.ratemin[DDIR_TRIM]) {
+		if (o->ratemin[DDIR_READ] || o->ratemin[DDIR_WRITE] ||
+				o->ratemin[DDIR_TRIM]) {
 		        memcpy(&td->lastrate[DDIR_READ], &td->bw_sample_time,
 						sizeof(td->bw_sample_time));
 		        memcpy(&td->lastrate[DDIR_WRITE], &td->bw_sample_time,
@@ -1302,8 +1324,8 @@ static void *thread_main(void *data)
 		if (td->error || td->terminate)
 			break;
 
-		if (!td->o.do_verify ||
-		    td->o.verify == VERIFY_NONE ||
+		if (!o->do_verify ||
+		    o->verify == VERIFY_NONE ||
 		    (td->io_ops->flags & FIO_UNIDIR))
 			continue;
 
@@ -1328,46 +1350,48 @@ static void *thread_main(void *data)
 	td->ts.io_bytes[DDIR_WRITE] = td->io_bytes[DDIR_WRITE];
 	td->ts.io_bytes[DDIR_TRIM] = td->io_bytes[DDIR_TRIM];
 
+	fio_unpin_memory(td);
+
 	fio_mutex_down(writeout_mutex);
 	if (td->bw_log) {
-		if (td->o.bw_log_file) {
+		if (o->bw_log_file) {
 			finish_log_named(td, td->bw_log,
-						td->o.bw_log_file, "bw");
+						o->bw_log_file, "bw");
 		} else
 			finish_log(td, td->bw_log, "bw");
 	}
 	if (td->lat_log) {
-		if (td->o.lat_log_file) {
+		if (o->lat_log_file) {
 			finish_log_named(td, td->lat_log,
-						td->o.lat_log_file, "lat");
+						o->lat_log_file, "lat");
 		} else
 			finish_log(td, td->lat_log, "lat");
 	}
 	if (td->slat_log) {
-		if (td->o.lat_log_file) {
+		if (o->lat_log_file) {
 			finish_log_named(td, td->slat_log,
-						td->o.lat_log_file, "slat");
+						o->lat_log_file, "slat");
 		} else
 			finish_log(td, td->slat_log, "slat");
 	}
 	if (td->clat_log) {
-		if (td->o.lat_log_file) {
+		if (o->lat_log_file) {
 			finish_log_named(td, td->clat_log,
-						td->o.lat_log_file, "clat");
+						o->lat_log_file, "clat");
 		} else
 			finish_log(td, td->clat_log, "clat");
 	}
 	if (td->iops_log) {
-		if (td->o.iops_log_file) {
+		if (o->iops_log_file) {
 			finish_log_named(td, td->iops_log,
-						td->o.iops_log_file, "iops");
+						o->iops_log_file, "iops");
 		} else
 			finish_log(td, td->iops_log, "iops");
 	}
 
 	fio_mutex_up(writeout_mutex);
-	if (td->o.exec_postrun)
-		exec_string(td->o.exec_postrun);
+	if (o->exec_postrun)
+		exec_string(o->exec_postrun);
 
 	if (exitall_on_terminate)
 		fio_terminate_threads(td->groupid);
@@ -1377,7 +1401,7 @@ err:
 		log_info("fio: pid=%d, err=%d/%s\n", (int) td->pid, td->error,
 							td->verror);
 
-	if (td->o.verify_async)
+	if (o->verify_async)
 		verify_async_exit(td);
 
 	close_and_free_files(td);
@@ -1385,8 +1409,8 @@ err:
 	close_ioengine(td);
 	cgroup_shutdown(td, &cgroup_mnt);
 
-	if (td->o.cpumask_set) {
-		int ret = fio_cpuset_exit(&td->o.cpumask);
+	if (o->cpumask_set) {
+		int ret = fio_cpuset_exit(&o->cpumask);
 
 		td_verror(td, ret, "fio_cpuset_exit");
 	}
@@ -1394,7 +1418,7 @@ err:
 	/*
 	 * do this very late, it will log file closing as well
 	 */
-	if (td->o.write_iolog_file)
+	if (o->write_iolog_file)
 		write_iolog_close(td);
 
 	fio_mutex_remove(td->rusage_sem);
@@ -1534,6 +1558,12 @@ reaped:
 		fio_terminate_threads(TERMINATE_ALL);
 }
 
+static void do_usleep(unsigned int usecs)
+{
+	check_for_running_stats();
+	usleep(usecs);
+}
+
 /*
  * Main function for kicking off and reaping jobs, as needed.
  */
@@ -1543,9 +1573,6 @@ static void run_threads(void)
 	unsigned long spent;
 	unsigned int i, todo, nr_running, m_rate, t_rate, nr_started;
 
-	if (fio_pin_memory())
-		return;
-
 	if (fio_gtod_offload && fio_start_gtod_thread())
 		return;
 	
@@ -1553,6 +1580,14 @@ static void run_threads(void)
 
 	set_sig_handlers();
 
+	nr_thread = nr_process = 0;
+	for_each_td(td, i) {
+		if (td->o.use_thread)
+			nr_thread++;
+		else
+			nr_process++;
+	}
+
 	if (output_format == FIO_OUTPUT_NORMAL) {
 		log_info("Starting ");
 		if (nr_thread)
@@ -1707,7 +1742,7 @@ static void run_threads(void)
 			if (mtime_since_now(&this_start) > JOB_START_TIMEOUT)
 				break;
 
-			usleep(100000);
+			do_usleep(100000);
 
 			for (i = 0; i < this_jobs; i++) {
 				td = map[i];
@@ -1758,27 +1793,18 @@ static void run_threads(void)
 
 		reap_threads(&nr_running, &t_rate, &m_rate);
 
-		if (todo) {
-			if (is_backend)
-				fio_server_idle_loop();
-			else
-				usleep(100000);
-		}
+		if (todo)
+			do_usleep(100000);
 	}
 
 	while (nr_running) {
 		reap_threads(&nr_running, &t_rate, &m_rate);
-
-		if (is_backend)
-			fio_server_idle_loop();
-		else
-			usleep(10000);
+		do_usleep(10000);
 	}
 
 	fio_idle_prof_stop();
 
 	update_io_ticks();
-	fio_unpin_memory();
 }
 
 void wait_for_disk_thread_exit(void)
@@ -1856,9 +1882,9 @@ int fio_backend(void)
 		return 0;
 
 	if (write_bw_log) {
-		setup_log(&agg_io_log[DDIR_READ], 0);
-		setup_log(&agg_io_log[DDIR_WRITE], 0);
-		setup_log(&agg_io_log[DDIR_TRIM], 0);
+		setup_log(&agg_io_log[DDIR_READ], 0, IO_LOG_TYPE_BW);
+		setup_log(&agg_io_log[DDIR_WRITE], 0, IO_LOG_TYPE_BW);
+		setup_log(&agg_io_log[DDIR_TRIM], 0, IO_LOG_TYPE_BW);
 	}
 
 	startup_mutex = fio_mutex_init(FIO_MUTEX_LOCKED);
@@ -1869,6 +1895,7 @@ int fio_backend(void)
 		return 1;
 
 	set_genesis_time();
+	stat_init();
 	create_disk_util_thread();
 
 	cgroup_list = smalloc(sizeof(*cgroup_list));
@@ -1898,5 +1925,6 @@ int fio_backend(void)
 	fio_mutex_remove(startup_mutex);
 	fio_mutex_remove(writeout_mutex);
 	fio_mutex_remove(disk_thread_mutex);
+	stat_exit();
 	return exit_value;
 }
