@@ -14,12 +14,10 @@
 #include <inttypes.h>
 #include <assert.h>
 
-struct thread_data;
-
 #include "compiler/compiler.h"
+#include "thread_options.h"
 #include "flist.h"
 #include "fifo.h"
-#include "rbtree.h"
 #include "arch/arch.h"
 #include "os/os.h"
 #include "mutex.h"
@@ -36,6 +34,8 @@ struct thread_data;
 #include "gettime.h"
 #include "lib/getopt.h"
 #include "lib/rand.h"
+#include "lib/rbtree.h"
+#include "client.h"
 #include "server.h"
 #include "stat.h"
 #include "flow.h"
@@ -54,17 +54,6 @@ struct thread_data;
 #define MPOL_LOCAL MPOL_MAX
 #endif
 
-/*
- * What type of allocation to use for io buffers
- */
-enum fio_memtype {
-	MEM_MALLOC = 0,	/* ordinary malloc */
-	MEM_SHM,	/* use shared memory segments */
-	MEM_SHMHUGE,	/* use shared memory segments with huge pages */
-	MEM_MMAP,	/* use anonynomous mmap */
-	MEM_MMAPHUGE,	/* memory mapped huge file */
-};
-
 /*
  * offset generator types
  */
@@ -73,240 +62,6 @@ enum {
 	RW_SEQ_IDENT,
 };
 
-/*
- * What type of errors to continue on when continue_on_error is used
- */
-enum error_type_bit {
-	ERROR_TYPE_READ_BIT = 0,
-	ERROR_TYPE_WRITE_BIT = 1,
-	ERROR_TYPE_VERIFY_BIT = 2,
-	ERROR_TYPE_CNT = 3,
-};
-
-enum error_type {
-        ERROR_TYPE_NONE = 0,
-        ERROR_TYPE_READ = 1 << ERROR_TYPE_READ_BIT,
-        ERROR_TYPE_WRITE = 1 << ERROR_TYPE_WRITE_BIT,
-        ERROR_TYPE_VERIFY = 1 << ERROR_TYPE_VERIFY_BIT,
-        ERROR_TYPE_ANY = 0xffff,
-};
-
-struct bssplit {
-	unsigned int bs;
-	unsigned char perc;
-};
-
-struct thread_options {
-	int pad;
-	char *description;
-	char *name;
-	char *directory;
-	char *filename;
-	char *opendir;
-	char *ioengine;
-	enum td_ddir td_ddir;
-	unsigned int rw_seq;
-	unsigned int kb_base;
-	unsigned int ddir_seq_nr;
-	long ddir_seq_add;
-	unsigned int iodepth;
-	unsigned int iodepth_low;
-	unsigned int iodepth_batch;
-	unsigned int iodepth_batch_complete;
-
-	unsigned long long size;
-	unsigned int size_percent;
-	unsigned int fill_device;
-	unsigned long long file_size_low;
-	unsigned long long file_size_high;
-	unsigned long long start_offset;
-
-	unsigned int bs[DDIR_RWDIR_CNT];
-	unsigned int ba[DDIR_RWDIR_CNT];
-	unsigned int min_bs[DDIR_RWDIR_CNT];
-	unsigned int max_bs[DDIR_RWDIR_CNT];
-	struct bssplit *bssplit[DDIR_RWDIR_CNT];
-	unsigned int bssplit_nr[DDIR_RWDIR_CNT];
-
-	int *ignore_error[ERROR_TYPE_CNT];
-	unsigned int ignore_error_nr[ERROR_TYPE_CNT];
-	unsigned int error_dump;
-
-	unsigned int nr_files;
-	unsigned int open_files;
-	enum file_lock_mode file_lock_mode;
-
-	unsigned int odirect;
-	unsigned int invalidate_cache;
-	unsigned int create_serialize;
-	unsigned int create_fsync;
-	unsigned int create_on_open;
-	unsigned int create_only;
-	unsigned int end_fsync;
-	unsigned int pre_read;
-	unsigned int sync_io;
-	unsigned int verify;
-	unsigned int do_verify;
-	unsigned int verifysort;
-	unsigned int verifysort_nr;
-	unsigned int verify_interval;
-	unsigned int verify_offset;
-	char verify_pattern[MAX_PATTERN_SIZE];
-	unsigned int verify_pattern_bytes;
-	unsigned int verify_fatal;
-	unsigned int verify_dump;
-	unsigned int verify_async;
-	unsigned long long verify_backlog;
-	unsigned int verify_batch;
-	unsigned int experimental_verify;
-	unsigned int use_thread;
-	unsigned int unlink;
-	unsigned int do_disk_util;
-	unsigned int override_sync;
-	unsigned int rand_repeatable;
-	unsigned int use_os_rand;
-	unsigned int write_lat_log;
-	unsigned int write_bw_log;
-	unsigned int write_iops_log;
-	unsigned int log_avg_msec;
-	unsigned int norandommap;
-	unsigned int softrandommap;
-	unsigned int bs_unaligned;
-	unsigned int fsync_on_close;
-
-	unsigned int random_distribution;
-	double zipf_theta;
-	double pareto_h;
-
-	unsigned int random_generator;
-
-	unsigned int hugepage_size;
-	unsigned int rw_min_bs;
-	unsigned int thinktime;
-	unsigned int thinktime_spin;
-	unsigned int thinktime_blocks;
-	unsigned int fsync_blocks;
-	unsigned int fdatasync_blocks;
-	unsigned int barrier_blocks;
-	unsigned long long start_delay;
-	unsigned long long timeout;
-	unsigned long long ramp_time;
-	unsigned int overwrite;
-	unsigned int bw_avg_time;
-	unsigned int iops_avg_time;
-	unsigned int loops;
-	unsigned long long zone_range;
-	unsigned long long zone_size;
-	unsigned long long zone_skip;
-	enum fio_memtype mem_type;
-	unsigned int mem_align;
-
-	unsigned int max_latency;
-
-	unsigned int stonewall;
-	unsigned int new_group;
-	unsigned int numjobs;
-	os_cpu_mask_t cpumask;
-	unsigned int cpumask_set;
-	os_cpu_mask_t verify_cpumask;
-	unsigned int verify_cpumask_set;
-#ifdef CONFIG_LIBNUMA
-	struct bitmask *numa_cpunodesmask;
-	unsigned int numa_cpumask_set;
-	unsigned short numa_mem_mode;
-	unsigned int numa_mem_prefer_node;
-	struct bitmask *numa_memnodesmask;
-	unsigned int numa_memmask_set;
-#endif
-	unsigned int iolog;
-	unsigned int rwmixcycle;
-	unsigned int rwmix[2];
-	unsigned int nice;
-	unsigned int file_service_type;
-	unsigned int group_reporting;
-	unsigned int fadvise_hint;
-	enum fio_fallocate_mode fallocate_mode;
-	unsigned int zero_buffers;
-	unsigned int refill_buffers;
-	unsigned int scramble_buffers;
-	unsigned int compress_percentage;
-	unsigned int compress_chunk;
-	unsigned int time_based;
-	unsigned int disable_lat;
-	unsigned int disable_clat;
-	unsigned int disable_slat;
-	unsigned int disable_bw;
-	unsigned int unified_rw_rep;
-	unsigned int gtod_reduce;
-	unsigned int gtod_cpu;
-	unsigned int gtod_offload;
-	enum fio_cs clocksource;
-	unsigned int no_stall;
-	unsigned int trim_percentage;
-	unsigned int trim_batch;
-	unsigned int trim_zero;
-	unsigned long long trim_backlog;
-	unsigned int clat_percentiles;
-	unsigned int percentile_precision;	/* digits after decimal for percentiles */
-	fio_fp64_t percentile_list[FIO_IO_U_LIST_MAX_LEN];
-
-	char *read_iolog_file;
-	char *write_iolog_file;
-	char *bw_log_file;
-	char *lat_log_file;
-	char *iops_log_file;
-	char *replay_redirect;
-
-	/*
-	 * Pre-run and post-run shell
-	 */
-	char *exec_prerun;
-	char *exec_postrun;
-
-	unsigned int rate[DDIR_RWDIR_CNT];
-	unsigned int ratemin[DDIR_RWDIR_CNT];
-	unsigned int ratecycle;
-	unsigned int rate_iops[DDIR_RWDIR_CNT];
-	unsigned int rate_iops_min[DDIR_RWDIR_CNT];
-
-	char *ioscheduler;
-
-	/*
-	 * CPU "io" cycle burner
-	 */
-	unsigned int cpuload;
-	unsigned int cpucycle;
-
-	/*
-	 * I/O Error handling
-	 */
-	enum error_type continue_on_error;
-
-	/*
-	 * Benchmark profile type
-	 */
-	char *profile;
-
-	/*
-	 * blkio cgroup support
-	 */
-	char *cgroup;
-	unsigned int cgroup_weight;
-	unsigned int cgroup_nodelete;
-
-	unsigned int uid;
-	unsigned int gid;
-
-	int flow_id;
-	int flow;
-	int flow_watermark;
-	unsigned int flow_sleep;
-
-	unsigned long long offset_increment;
-
-	unsigned int sync_file_range;
-};
-
 enum {
 	TD_F_VER_BACKLOG	= 1,
 	TD_F_TRIM_BACKLOG	= 2,
@@ -326,6 +81,7 @@ enum {
 	FIO_RAND_FILE_SIZE_OFF,
 	FIO_RAND_TRIM_OFF,
 	FIO_RAND_BUF_OFF,
+	FIO_RAND_SEQ_RAND_OFF,
 	FIO_RAND_NR_OFFS,
 };
 
@@ -338,10 +94,12 @@ struct thread_data {
 	void *eo;
 	char verror[FIO_VERROR_SIZE];
 	pthread_t thread;
-	int thread_number;
-	int groupid;
+	unsigned int thread_number;
+	unsigned int groupid;
 	struct thread_stat ts;
 
+	int client_type;
+
 	struct io_log *slat_log;
 	struct io_log *clat_log;
 	struct io_log *lat_log;
@@ -379,12 +137,9 @@ struct thread_data {
 	size_t orig_buffer_size;
 	volatile int terminate;
 	volatile int runstate;
-	unsigned int ioprio;
-	unsigned int ioprio_set;
 	unsigned int last_was_sync;
 	enum fio_ddir last_ddir;
 
-	char *mmapfile;
 	int mmapfd;
 
 	void *iolog_buf;
@@ -501,6 +256,14 @@ struct thread_data {
 	enum fio_ddir rwmix_ddir;
 	unsigned int ddir_seq_nr;
 
+	/*
+	 * rand/seq mixed workload state
+	 */
+	union {
+		os_random_state_t seq_rand_state;
+		struct frand_state __seq_rand_state;
+	};
+
 	/*
 	 * IO history logs for verification. We use a tree for sorting,
 	 * if we are overwriting. Otherwise just use a fifo.
@@ -552,6 +315,8 @@ struct thread_data {
 	 */
 	struct prof_io_ops prof_io_ops;
 	void *prof_data;
+
+	void *pinned_mem;
 };
 
 /*
@@ -565,9 +330,9 @@ enum {
 
 #define __td_verror(td, err, msg, func)					\
 	do {								\
+		int e = (err);						\
 		if ((td)->error)					\
 			break;						\
-		int e = (err);						\
 		(td)->error = e;					\
 		if (!(td)->first_error)					\
 			snprintf(td->verror, sizeof(td->verror), "file:%s:%d, func=%s, error=%s", __FILE__, __LINE__, (func), (msg));		\
@@ -587,12 +352,10 @@ enum {
 extern int exitall_on_terminate;
 extern unsigned int thread_number;
 extern unsigned int stat_number;
-extern unsigned int nr_process, nr_thread;
 extern int shm_id;
 extern int groupid;
 extern int output_format;
 extern int temp_stall_ts;
-extern unsigned long long mlock_size;
 extern uintptr_t page_mask, page_size;
 extern int read_only;
 extern int eta_print;
@@ -608,6 +371,7 @@ extern int terse_version;
 extern int is_backend;
 extern int nr_clients;
 extern int log_syslog;
+extern int status_interval;
 extern const char fio_version_string[];
 
 extern struct thread_data *threads;
@@ -619,41 +383,6 @@ static inline void fio_ro_check(struct thread_data *td, struct io_u *io_u)
 
 #define REAL_MAX_JOBS		2048
 
-static inline enum error_type_bit td_error_type(enum fio_ddir ddir, int err)
-{
-	if (err == EILSEQ)
-		return ERROR_TYPE_VERIFY_BIT;
-	if (ddir == DDIR_READ)
-		return ERROR_TYPE_READ_BIT;
-	return ERROR_TYPE_WRITE_BIT;
-}
-
-static int __NON_FATAL_ERR[] = {EIO, EILSEQ};
-static inline int td_non_fatal_error(struct thread_data *td,
-				     enum error_type_bit etype, int err)
-{
-	int i;
-	if (!td->o.ignore_error[etype]) {
-		td->o.ignore_error[etype] = __NON_FATAL_ERR;
-		td->o.ignore_error_nr[etype] = sizeof(__NON_FATAL_ERR)
-			/ sizeof(int);
-	}
-
-	if (!(td->o.continue_on_error & (1 << etype)))
-		return 0;
-	for (i = 0; i < td->o.ignore_error_nr[etype]; i++)
-		if (td->o.ignore_error[etype][i] == err)
-			return 1;
-	return 0;
-}
-
-static inline void update_error_count(struct thread_data *td, int err)
-{
-	td->total_err_count++;
-	if (td->total_err_count == 1)
-		td->first_error = err;
-}
-
 static inline int should_fsync(struct thread_data *td)
 {
 	if (td->last_was_sync)
@@ -667,9 +396,10 @@ static inline int should_fsync(struct thread_data *td)
 /*
  * Init/option functions
  */
+extern int __must_check fio_init_options(void);
 extern int __must_check parse_options(int, char **);
-extern int parse_jobs_ini(char *, int, int);
-extern int parse_cmd_line(int, char **);
+extern int parse_jobs_ini(char *, int, int, int);
+extern int parse_cmd_line(int, char **, int);
 extern int fio_backend(void);
 extern void reset_fio_state(void);
 extern void clear_io_state(struct thread_data *);
@@ -684,10 +414,14 @@ extern void fio_options_dup_and_init(struct option *);
 extern void fio_options_mem_dupe(struct thread_data *);
 extern void options_mem_dupe(void *data, struct fio_option *options);
 extern void td_fill_rand_seeds(struct thread_data *);
-extern void add_job_opts(const char **);
-extern char *num2str(unsigned long, int, int, int);
+extern void add_job_opts(const char **, int);
+extern char *num2str(unsigned long, int, int, int, int);
 extern int ioengine_load(struct thread_data *);
 
+extern uintptr_t page_mask;
+extern uintptr_t page_size;
+extern int initialize_fio(char *envp[]);
+
 #define FIO_GETOPT_JOB		0x89000000
 #define FIO_GETOPT_IOENGINE	0x98000000
 #define FIO_NR_OPTIONS		(FIO_MAX_OPTS + 128)
@@ -697,6 +431,7 @@ extern int ioengine_load(struct thread_data *);
  */
 extern void print_thread_status(void);
 extern void print_status_init(int);
+extern char *fio_uint_to_kmg(unsigned int val);
 
 /*
  * Thread life cycle. Once a thread has a runstate beyond TD_INITIALIZED, it
@@ -725,10 +460,11 @@ extern void fio_terminate_threads(int);
 /*
  * Memory helpers
  */
-extern int __must_check fio_pin_memory(void);
-extern void fio_unpin_memory(void);
+extern int __must_check fio_pin_memory(struct thread_data *);
+extern void fio_unpin_memory(struct thread_data *);
 extern int __must_check allocate_io_mem(struct thread_data *);
 extern void free_io_mem(struct thread_data *);
+extern void free_threads_shm(void);
 
 /*
  * Reset stats after ramp time completes
@@ -743,13 +479,6 @@ extern int is_blktrace(const char *);
 extern int load_blktrace(struct thread_data *, const char *);
 #endif
 
-/*
- * Mark unused variables passed to ops functions as unused, to silence gcc
- */
-#define fio_unused	__attribute((__unused__))
-#define fio_init	__attribute__((constructor))
-#define fio_exit	__attribute__((destructor))
-
 #define for_each_td(td, i)	\
 	for ((i) = 0, (td) = &threads[0]; (i) < (int) thread_number; (i)++, (td)++)
 #define for_each_file(td, f, i)	\
@@ -815,6 +544,14 @@ static inline unsigned int td_max_bs(struct thread_data *td)
 	return max(td->o.max_bs[DDIR_TRIM], max_bs);
 }
 
+static inline unsigned int td_min_bs(struct thread_data *td)
+{
+	unsigned int min_bs;
+
+	min_bs = min(td->o.min_bs[DDIR_READ], td->o.min_bs[DDIR_WRITE]);
+	return min(td->o.min_bs[DDIR_TRIM], min_bs);
+}
+
 static inline int is_power_of_2(unsigned int val)
 {
 	return (val != 0 && ((val & (val - 1)) == 0));
@@ -845,6 +582,8 @@ static inline void td_io_u_free_notify(struct thread_data *td)
 extern const char *fio_get_arch_string(int);
 extern const char *fio_get_os_string(int);
 
+#define ARRAY_SIZE(x) (sizeof((x)) / (sizeof((x)[0])))
+
 enum {
 	FIO_OUTPUT_TERSE	= 0,
 	FIO_OUTPUT_JSON,
